(ns tservice.plugins.xps2pdf.xps
  "XPS Convertors, e.g. xps2png, xps2jpg, xps2pdf etc."
  (:require [clojure.java.shell :as shell]
            [tservice.lib.files :refer [get-plugin-dir get-path-variable]]
            [tservice.lib.fs :as fs]
            [clojure.tools.logging :as log]
            [clojure.string :as str]))

(defn add-env-to-path
  []
  (let [env-bin-path (fs/join-paths (get-plugin-dir) "xps2pdf" "bin")
        path (get-path-variable)]
    (str env-bin-path ":" path)))

;; Need to install libgxps tools
;; For Mac, `brew install libgxps`, more details in https://formulae.brew.sh/formula/libgxps
;; For Linux, `yum install libgxps-tools`
(defn exist-bin
  [bin]
  (shell/with-sh-env {:PATH   (add-env-to-path)
                      :LC_ALL "en_US.utf-8"
                      :LANG   "en_US.utf-8"}
    (let [command ["sh" "-c"
                   (format "which %s" bin)]
          result  (apply shell/sh command)]
      (if (= (:exit result) 0) true false))))

(defn- convertor
  "Common convertor dispather.
   Bug fixes: sh is implemented using Clojure futures.  See examples for 'future'
   for discussion of an undesirable 1-minute wait that can occur before
   your standalone Clojure program exits if you do not use shutdown-agents.
   More details: https://clojuredocs.org/clojure.java.shell/sh#example-542692d6c026201cdc3270e7

   Fix Bug: 
     MUST NOT use shutdown-agents, it will cause the problem which the FutureTask was not completed but agent is down.'
  "
  [cmd from to]
  (try
    (shell/sh cmd from to)
    (catch Exception e
      (if (exist-bin cmd)
        (log/error (str "Exception: " (.toString e)))
        (log/warn "You need to install libgxps-tools for xps2pdf plugin. For Linux, `yum install libgxps-tools`.")))))

(defn xps2png
  "Convert XPS to PDF."
  [from to]
  (convertor "xpstopng" from to))

(defn xps2pdf
  "Convert XPS to PNG."
  [from to]
  (convertor "xpstopdf" from to))

(defn xps2jpeg
  "Convert XPS to JPEG."
  [from to]
  (convertor "xpstojpeg" from to))

(defn xps2svg
  "Convert XPS to SVG."
  [from to]
  (convertor "xpstosvg" from to))

(def command-map
  {:pdf xps2pdf
   :svg xps2svg
   :jpeg xps2jpeg
   :png xps2png})

(defn command
  "Dispatch to different function."
  [{:keys [input output type]}]
  (println "Convert XPS to" (str/upper-case type) ":" input)
  (((keyword type) command-map) input output))